/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LayeredPane;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableRootEditPart;

/**
 * This class is an extension to the {@link ScalableRootEditPart}. It provides
 * four additional printable layers: the node layer, the label layer, the
 * decoration layer and the custom layer.
 * <P>
 * The node layer is between the primary layer and the connection layer defined
 * by {@link ScalableRootEditPart}. The label, decoration and custom layer are
 * above the connection layer.
 * <P>
 * Typically graphical figures like boxes and ellipses are displayed in the node
 * layer. Labels are displayed in the label layer. Markers and decorations are
 * shown in the decoration layer. Finally, the custom layer may be used to
 * display further information.
 * <P>
 * If an edit part wants to display information below all other layers (but of
 * course above the grid layer), it can use the
 * {@link LayerConstants#PRIMARY_LAYER}.
 * <P>
 * If an edit part wants to display information above the decorations, it can
 * use the {@link #CUSTOM_LAYER}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 6AE34957657F23D48142F8C130CCC547
 */
public class ExtendedLayerRootEditPart extends ScalableRootEditPart {
	/** The node layer ID. */
	public static final String NODE_LAYER = ExtendedLayerRootEditPart.class
			.getName() + ".nodeLayer";

	/** The label layer ID. */
	public static final String LABEL_LAYER = ExtendedLayerRootEditPart.class
			.getName() + ".labelLayer";

	/** The decoration layer ID. */
	public static final String DECORATION_LAYER = ExtendedLayerRootEditPart.class
			.getName() + ".decorationLayer";

	/** The custom layer ID. */
	public static final String CUSTOM_LAYER = ExtendedLayerRootEditPart.class
			.getName() + ".customLayer";

	/** Stores the node and connector layer. */
	private Layer nodeLayer;

	/** Stores the label layer. */
	private Layer labelLayer;

	/** Stores the decoration layer. */
	private Layer decorationLayer;

	/** Stores the custom layer. */
	private Layer customLayer;

	/** {@inheritDoc} */
	@Override
	protected LayeredPane createPrintableLayers() {
		LayeredPane pane = super.createPrintableLayers();

		nodeLayer = createDefaultLayer();
		labelLayer = createDefaultLayer();
		decorationLayer = createDefaultLayer();
		customLayer = createDefaultLayer();

		pane.addLayerBefore(nodeLayer, NODE_LAYER, PRIMARY_LAYER);
		pane.addLayerBefore(labelLayer, LABEL_LAYER, CONNECTION_LAYER);
		pane.addLayerBefore(decorationLayer, DECORATION_LAYER, LABEL_LAYER);
		pane.addLayerBefore(customLayer, CUSTOM_LAYER, DECORATION_LAYER);

		return pane;
	}

	/** {@inheritDoc} */
	@Override
	public IFigure getContentPane() {
		return getLayer(NODE_LAYER);
	}

	/** {@inheritDoc} */
	@Override
	public IFigure getLayer(Object key) {
		if (key == NODE_LAYER) {
			return nodeLayer;
		}
		if (key == LABEL_LAYER) {
			return labelLayer;
		}
		if (key == DECORATION_LAYER) {
			return decorationLayer;
		}
		if (key == CUSTOM_LAYER) {
			return customLayer;
		}
		return super.getLayer(key);
	}

	/** Creates the default layer with XYLayout. */
	private Layer createDefaultLayer() {
		Layer l = new FreeformLayer() {
			/** {@inheritDoc} */
			@Override
			public void paint(Graphics graphics) {
				// TODO Auto-generated method stub
				super.paint(graphics);
			}

			/** {@inheritDoc} */
			@Override
			protected void paintChildren(Graphics graphics) {
				// TODO Auto-generated method stub
				super.paintChildren(graphics);
			}
		};
		l.setOpaque(false);
		l.setLayoutManager(new XYLayout());
		l.setBackgroundColor(ColorConstants.white);
		return l;
	}

}
